@@ -34,11 +34,11 @@ namespace Mono.Debugging.Backend.Mdb
 	{
 		string InitializeMdb (string mdbVersion, int mdPid);
 		
-		void Run (MonoDebuggerStartInfo startInfo);
+		void Run (MonoDebuggerStartInfo startInfo, DebuggerSessionOptions options);
 
 		void Stop ();
 		
-		void AttachToProcess (long id);
+		void AttachToProcess (long id, DebuggerSessionOptions options);
 		
 		void Detach ();
 		
@@ -74,7 +74,7 @@ namespace Mono.Debugging.Backend.Mdb
 		
 		ThreadInfo[] GetThreads (long processId);
 		
-		ProcessInfo[] GetPocesses ();
+		ProcessInfo[] GetProcesses ();
 		
 		Backtrace GetThreadBacktrace (long processId, long threadId);
 		
