@@ -25,6 +25,7 @@
 // THE SOFTWARE.
 
 using System;
+using System.Linq;
 using System.Text;
 using System.Collections;
 using ST = System.Threading;
@@ -298,6 +299,15 @@ namespace DebuggerServer
 			else
 				return null;
 		}
+		
+		public override ValueReference GetCurrentException (EvaluationContext gctx)
+		{
+			MdbEvaluationContext ctx = (MdbEvaluationContext) gctx;
+			if (ctx.Exception != null)
+				return LiteralValueReference.CreateTargetObjectLiteral (ctx, ctx.Options.CurrentExceptionTag, ctx.Exception);
+			else
+				return null;
+		}
 
 
 		public override IEnumerable<ValueReference> GetParameters (EvaluationContext gctx)
@@ -452,10 +462,12 @@ namespace DebuggerServer
 		}
 
 
-		public override ValueReference GetIndexerReference (EvaluationContext gctx, object target, object index)
+		public override ValueReference GetIndexerReference (EvaluationContext gctx, object target, object[] indices)
 		{
 			MdbEvaluationContext ctx = (MdbEvaluationContext) gctx;
-			return IndexerValueReference.CreateIndexerValueReference (ctx, (TargetObject) target, (TargetObject) index);
+			TargetObject[] indexArray = new TargetObject [indices.Length];
+			Array.Copy (indices, indexArray, indices.Length);
+			return IndexerValueReference.CreateIndexerValueReference (ctx, (TargetObject) target, indexArray);
 		}
 
 
@@ -525,7 +537,7 @@ namespace DebuggerServer
 				return ObjectValue.CreateObject (null, path, "", "", flags | ObjectValueFlags.ReadOnly, null);
 
 			if (obj.HasAddress && obj.GetAddress (ctx.Thread).IsNull)
-				return ObjectValue.CreateObject (null, path, obj.TypeName, ctx.Evaluator.ToExpression (null), flags, null);
+				return ObjectValue.CreateObject (null, path, obj.TypeName, ctx.Evaluator.ToExpression (ctx, null), flags, null);
 			
 			switch (obj.Kind) {
 				
@@ -556,7 +568,7 @@ namespace DebuggerServer
 					}
 					else {
 						flags |= ObjectValueFlags.Primitive;
-						return ObjectValue.CreateObject (source, path, obj.TypeName, ctx.Evaluator.ToExpression (null), flags, new ObjectValue [0]);
+						return ObjectValue.CreateObject (source, path, obj.TypeName, ctx.Evaluator.ToExpression (ctx, null), flags, new ObjectValue [0]);
 					}
 				default:
 					return ObjectValue.CreateFatalError (path.LastName, "Unknown value type: " + obj.Kind, flags);
